-------------
-- Awesome --
-------------






------ REQUIREMENTS ------ 





----- LuaRocks -----

-- If LuaRocks is installed, make sure that packages installed through it are
-- found (e.g. lgi). If LuaRocks is not installed, do nothing.
pcall(require, "luarocks.loader")





----- LIBRARIES -----




---- Standard awesome library ----

local gears = require("gears")
local awful = require("awful")

require("awful.autofocus")




---- Widget and Layout Library ----

local wibox = require("wibox")




---- Theme Handling Library ----

local beautiful = require("beautiful")




---- Shapes Library ----

shape = require("gears.shape")




---- Xresources Library ----

local xresources = require("beautiful.xresources")




---- DPI Library ----

local dpi = xresources.apply_dpi




---- Notification Library ----
 
local naughty = require("naughty")
local menubar = require("menubar")
local hotkeys_popup = require("awful.hotkeys_popup")

-- Enable hotkeys help widget for VIM and other apps
-- when client with a matching name is opened:
require("awful.hotkeys_popup.keys")






------ ERROR HANDLING ------




---- Check And Fallback ----

-- Check if awesome encountered an error during startup and fell back to
-- another config (This code will only ever execute for the fallback config#)
if awesome.startup_errors then

    naughty.notify({ 
	             
	             preset = naughty.config.presets.critical,
                     title = "Oops, there were errors during startup!",
                     text = awesome.startup_errors })

end





---- Handle Errors ----

-- Handle runtime errors after startup
do
    local in_error = false
    awesome.connect_signal("debug::error", function (err)
        
	-- Make sure we don't go into an endless error loop
        if in_error then return end
        in_error = true
        
	-- Notify If An Error Happens
        naughty.notify({

		         preset = naughty.config.presets.critical,
                         title = "Oops, an error happened!",
                         text = tostring(err) 

		      })

        in_error = false

    end)

end






------ VARIABLE DEFINITIONS ------




---- Theme Dircetory ----

-- Themes define colours, icons, font and wallpapers.
beautiful.init(gears.filesystem.get_configuration_dir() .. "catppuccin.lua")





---- Applications ----

-- This is used later as the default terminal and editor to run.
terminal = "kitty"
editor = os.getenv("EDITOR") or "nano"
editor_cmd = terminal .. " -e " .. editor





---- Default Modkey ----

-- Usually, Mod4 is the key with a logo between Control and Alt.
-- If you do not like this or do not have such a key,
-- I suggest you to remap Mod4 to another key using xmodmap or other tools.
-- However, you can use another modifier like Mod1, but it may interact with others.
modkey = "Mod1"





---- Table Of Layouts ----

-- Table of layouts to cover with awful.layout.inc, order matters.
awful.layout.layouts = {



    --- Enabled ---
    
    --awful.layout.suit.mirror,
    awful.layout.suit.tile,
    awful.layout.suit.floating,
    awful.layout.suit.max,
    awful.layout.suit.max.fullscreen,



    --- Disabled ---

    --awful.layout.suit.tile.left,
    --awful.layout.suit.tile.bottom,
    --awful.layout.suit.tile.top,
    --awful.layout.suit.fair,
    --awful.layout.suit.fair.horizontal,
    --awful.layout.suit.spiral,
    --awful.layout.suit.spiral.dwindle,
    --awful.layout.suit.magnifier,
    --awful.layout.suit.corner.nw,
    --awful.layout.suit.corner.ne,
    --awful.layout.suit.corner.sw,
    --awful.layout.suit.corner.se,    
    
    
    
}






------ MENU ------




---- Awesome Menu ----

-- Create a launcher widget and a main menu
myawesomemenu = {

   { "hotkeys", function() hotkeys_popup.show_help(nil, awful.screen.focused()) end },
   { "manual", terminal .. " -e man awesome" },
   { "edit config", editor_cmd .. " " .. awesome.conffile },
   { "restart", awesome.restart },
   { "quit", function() awesome.quit() end },

}




--- Main Menu ----

mymainmenu = awful.menu({ items = {

	                            { "awesome", myawesomemenu, beautiful.awesome_icon },
                                    { "open terminal", terminal }

                                  }

                        })




---- Launcher ----

mylauncher = awful.widget.launcher(

                                     {

				     image = gears.filesystem.get_configuration_dir() .. "images/1544x1544_circle.png",
                                     menu = mymainmenu,
				     clip_shape = gears.shape.circle

			     	     }
				  
				  )




---- Menubar Configuration ----

menubar.utils.terminal = terminal -- Set the terminal for applications that require it






----- WIBAR ------




---- Status ----

-- Keyboard map indicator and switcher
mykeyboardlayout = awful.widget.keyboardlayout()

-- Create a textclock widget
mytextclock = wibox.widget.textclock()




---- Tags ----

awful.screen.connect_for_each_screen(function(s)



        --- Labels ---
    
        -- Each screen has its own tag table.
        awful.tag({ "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20" }, s, awful.layout.layouts[1])

        -- Create a taglist widget
        s.mytaglist = awful.widget.taglist {
        screen  = s,
        filter  = awful.widget.taglist.filter.all,
        buttons = taglist_buttons
        }

        -- Create a tasklist widget
        s.mytasklist = awful.widget.tasklist {
        screen  = s,
        filter  = awful.widget.tasklist.filter.currenttags,
        buttons = tasklist_buttons
    }

    


    --- Prompt Box --- 

    -- Create a promptbox for each screen
    s.mypromptbox = awful.widget.prompt()



    --- Layout Indicator ---

    -- Create an imagebox widget which will contain an icon indicating which layout we're using.
    -- We need one layoutbox per screen.
    s.mylayoutbox = awful.widget.layoutbox(s)





    ---- Wibox ----  
    


    --- Create The Wibox --- 
    
    function rounded_rect(cr, width, height)

    local RADIUS = 20 -- If you want to change how round the corners are, just change the RADIUS value.

    gears.shape.rounded_rect(cr, width, height, RADIUS)

    end

    s.mywibox = awful.wibar { 
	    position = "top", 
	    screen = s, 
	    shape = rounded_rect, 
	    margins = {
            left    = 5,
	    right   = 5
            }
	}



    -- Add widgets to the wibox
    s.mywibox:setup {

        layout = wibox.layout.align.horizontal,

        expand = "none",



	--- Left Widgets ---
	
        { 
	
	layout = wibox.layout.align.horizontal,

            mylauncher,

            s.mytaglist,
            s.mypromptbox,   
            s.mylayoutbox

        },



        --- Middle Widgests ---

        --s.mytasklist,

	{

            layout = wibox.layout.background,

            mytextclock,
	
	    --fg = text,
	    --bg = base .. "b3",

	    widget = wibox.container.background 

        },
        


        --- Right Widgets ---

        {

--[[
	    -- GPU --
	    -- Cannot add in addition to other widgets due to bug :( --

	    {

		{

		    {
                    
		        -- Margin on the left for arrow
			
		        {
 
                            layout = wibox.layout.background,
 
	                    left  = 25,
            	
	                    widget = wibox.container.margin

		        },

		        -- GPU Icon
			
		        {

     
                             layout = wibox.layout.background,
 
    			     image  = '~/.local/share/icons/gpu-card.svg',
  			     resize = false,
  		   	     widget = wibox.widget.imagebox
   
  		        },
   
			-- GPU Frequency Output
                        {

	                     layout = wibox.layout.background,

                             widget = awful.widget.watch('bash -c \'echo "$(cat /sys/class/drm/card0/gt_cur_freq_mhz) MHz "\'' , 2),  
                
	                }, 
                     
		        layout = wibox.layout.align.horizontal,

		    -- Formating -- text color, background color, shape... 
	            },

                    fg     = text,
	            bg     = green,

                    shape = function(cr, width, height)
	    	        gears.shape.rectangular_tag(cr, width, height, 15)
	            end,

                    widget = wibox.container.background  

	        },

	   
	    -- Formatting -- Layout, second background layer...
            layout = wibox.layout.background,

       	    widget = wibox.container.background, 

            bg = sapphire 

	    },

--]]

	    -- CPU --

	    {

		{

		    {
                    
		        -- Margin on the left for arrow
		        {

     
                            layout = wibox.layout.background,
 
	                    left  = 25,
            	
	                    widget = wibox.container.margin

		        },

		        -- CPU Icon
		        {

     
                             layout = wibox.layout.background,
 
		             markup = " ",
    		             halign = "center",
   		             valign = "center",
			     font = "feather-font-addon 15", 

    		             widget = wibox.widget.textbox
   
  		        },
   
		        -- CPU Usage Output
                        {

	                     layout = wibox.layout.background,

                             widget = awful.widget.watch('bash -c "ps aux --sort=-%cpu | awk \'NR==2 {print $3\\"% \\"}\'"' , 2),  
                
	                }, 
                     
		        layout = wibox.layout.align.horizontal,

		    -- Formating -- text color, background color, shape... 
	            },

                    fg     = base .. "b3",
	            bg     = yellow,

                    shape = function(cr, width, height)
	    	        gears.shape.rectangular_tag(cr, width, height, 15)
	            end,

                    widget = wibox.container.background  

	        },

	   
	    -- Formatting -- Layout, second background layer...
            layout = wibox.layout.background,

       	    widget = wibox.container.background, 

            --bg = green 

	    },


            -- RAM -- 

	    {

		{

		    {
                    
		        -- Margin on the left for arrow
		        {

     
                            layout = wibox.layout.background,
 
	                    left  = 25,
            	
	                    widget = wibox.container.margin

		        },

		        -- RAM Icon
		        {

     
                             layout = wibox.layout.background,
 
		             markup = " ",
    		             halign = "center",
   		             valign = "center",
			     font = "feather-font-addon 15", 

    		             widget = wibox.widget.textbox
   
  		        },
   
		        -- RAM Usage Output
                        {

	                     layout = wibox.layout.background,

                             widget = awful.widget.watch('bash -c "free -m | awk \'NR==2 {print $3 \\" MB \\"}\'"' , 2),  
                
	                }, 
                     
		        layout = wibox.layout.align.horizontal,

		    -- Formating -- text color, background color, shape... 
	            },

                    fg     = text,
	            bg     = peach,

                    shape = function(cr, width, height)
	    	        gears.shape.rectangular_tag(cr, width, height, 15)
	            end,

                    widget = wibox.container.background  

	        },

	   
	    -- Formatting -- Layout, second background layer...
            layout = wibox.layout.background,

       	    widget = wibox.container.background, 

            bg = yellow 

  	    },


	    -- Keybord --

            {

		{

		    -- Margin on the left for arrow
		    -- Keybord module
       	            {		

                        layout = wibox.layout.background,
            
	                widget = wibox.widget.textbox,
                        mykeyboardlayout,
                        --wibox.widget.systray(), 

	                left  = 15,
            	
	                widget = wibox.container.margin

	            },

                    -- Formating -- text color, background color, shape... 
                    fg     = text,
	            bg     = red,

                    shape = function(cr, width, height)
	    	        gears.shape.rectangular_tag(cr, width, height, 15)
	            end,

                    widget = wibox.container.background  

    	        },  
	   
	    -- Formatting -- Layout, second background layer...
            layout = wibox.layout.background,

       	    widget = wibox.container.background, 

            bg = peach

	    },


	layout = wibox.layout.align.horizontal

        },
     
    }

end)




----- WALLPAPER -----



--- Create Function to Set Wallpaper ---

local function set_wallpaper(s)

    -- Wallpaper
    if beautiful.wallpaper then

        local wallpaper = beautiful.wallpaper

        -- If wallpaper is a function, call it with the screen
        if type(wallpaper) == "function" then
            wallpaper = wallpaper(s)
        end

        gears.wallpaper.maximized(wallpaper, s, true)

    end

end



--- Run Function On Screen Geometry Change ---

-- Re-set wallpaper when a screen's geometry changes (e.g. different resolution)
screen.connect_signal("property::geometry", set_wallpaper)



--- Run Function At Startup ---

awful.screen.connect_for_each_screen(function(s)

    -- Wallpaper
    set_wallpaper(s)




    
------ MOUSE BINDINGS ------ 


---- Tags ----

root.buttons(gears.table.join(

    -- Togle menu
    awful.button({ }, 3, function () mymainmenu:toggle() end),

    awful.button({ }, 4, awful.tag.viewnext),

    awful.button({ }, 5, awful.tag.viewprev)

))

-- Create a wibox for each screen and add it
local taglist_buttons = gears.table.join(
                   
                    awful.button({ }, 1, function(t) t:view_only() end),

                    awful.button({ modkey }, 1, function(t)

                                              if client.focus then
                                                  client.focus:move_to_tag(t)
                                              end
                                          end),

                    awful.button({ }, 3, awful.tag.viewtoggle),

                    awful.button({ modkey }, 3, function(t)
                                              if client.focus then
                                                  client.focus:toggle_tag(t)
                                              end
                                          end),

                    awful.button({ }, 4, function(t) awful.tag.viewnext(t.screen) end),

                    awful.button({ }, 5, function(t) awful.tag.viewprev(t.screen) end)

)



--- Tasks ---

local tasklist_buttons = gears.table.join(

                     awful.button({ }, 1, function (c)
                                              if c == client.focus then
                                                  c.minimized = true
                                              else
                                                  c:emit_signal(
                                                      "request::activate",
                                                      "tasklist",
                                                      {raise = true}
                                                  )
                                              end
                                          end),

                     awful.button({ }, 3, function()
                                              awful.menu.client_list({ theme = { width = 250 } })
                                          end),

                     awful.button({ }, 4, function ()
                                              awful.client.focus.byidx(1)
                                          end),

                     awful.button({ }, 5, function ()
                                              awful.client.focus.byidx(-1)
                                          end))

end)




---- Layouts ----

awful.screen.connect_for_each_screen(function(s)

s.mylayoutbox = awful.widget.layoutbox(s)

s.mylayoutbox:buttons(gears.table.join(
                        awful.button({ }, 1, function () awful.layout.inc( 1) end),
                        awful.button({ }, 3, function () awful.layout.inc(-1) end),
                        awful.button({ }, 4, function () awful.layout.inc( 1) end),
                        awful.button({ }, 5, function () awful.layout.inc(-1) end)
			))

end)


---- Resizing ----

clientbuttons = gears.table.join(

    awful.button({ }, 1, function (c)
        c:emit_signal("request::activate", "mouse_click", {raise = true})
    end),

    awful.button({ modkey }, 1, function (c)
        c:emit_signal("request::activate", "mouse_click", {raise = true})
        awful.mouse.client.move(c)
    end),

    awful.button({ modkey }, 3, function (c)
        c:emit_signal("request::activate", "mouse_click", {raise = true})
        awful.mouse.client.resize(c)
    end)

)






------ KEYBINDINGS ------

globalkeys = gears.table.join(






    ----- Awseome -----

    -- Show help
    awful.key({ modkey,           }, "s",      hotkeys_popup.show_help,
              {description = "show help", group = "awesome"}),

    -- Show main menu
    awful.key({ modkey,           }, "w", function () mymainmenu:show() end,
              {description = "show main menu", group = "awesome"}),

    -- Reload awesome
    awful.key({ modkey, "Control" }, "r", awesome.restart,
              {description = "reload awesome", group = "awesome"}),
   
    -- Quit awesome
    awful.key({ modkey, "Shift"   }, "x", awesome.quit,
              {description = "quit awesome", group = "awesome"}),

    -- lua execute prompt  
    awful.key({ modkey }, "l",
              function ()
                  awful.prompt.run {
                    prompt       = "Run Lua code: ",
                    textbox      = awful.screen.focused().mypromptbox.widget,
                    exe_callback = awful.util.eval,
                    history_path = awful.util.get_cache_dir() .. "/history_eval"
                  }
              end,
              {description = "lua execute prompt", group = "awesome"}),






    ----- Layout -----
    


    --- Width factor (mwfact) ---
    
    -- Increase master width factor
    awful.key({ modkey,           }, "n",     function () awful.tag.incmwfact( 0.05) end,
              {description = "increase master width factor", group = "layout"}),

    -- Decrease master width factor
    awful.key({ modkey,           }, "d",     function () awful.tag.incmwfact(-0.05) end,
              {description = "decrease master width factor", group = "layout"}),



    --- Number Of Master Clients (nmaster) ---
    
    -- Increase the number of master clients 
    awful.key({ modkey, "Shift"   }, "d",     function () awful.tag.incnmaster( 1, nil, true) end,
              {description = "increase the number of master clients", group = "layout"}),

    -- Decrease the number of master clients
    awful.key({ modkey, "Shift"   }, "n",     function () awful.tag.incnmaster(-1, nil, true) end,
              {description = "decrease the number of master clients", group = "layout"}),

 

    --- Number Of Columns (col) ---
    
    -- Increase the number of columns
    awful.key({ modkey, "Control" }, "d",     function () awful.tag.incncol( 1, nil, true) end,
              {description = "increase the number of columns", group = "layout"}),

    -- Decrease the number of columns 
    awful.key({ modkey, "Control" }, "n",     function () awful.tag.incncol(-1, nil, true) end,
              {description = "decrease the number of columns", group = "layout"}),



    --- Selection ---

    -- Select next 
    awful.key({ modkey,           }, "space", function () awful.layout.inc( 1) end,
              {description = "select next", group = "layout"}),


    -- Select previous
    awful.key({ modkey, "Shift"   }, "space", function () awful.layout.inc(-1) end,
              {description = "select previous", group = "layout"})



)





    ----- Client -----
    
clientkeys = gears.table.join(
    
    -- Close
    awful.key({ modkey,           }, "k",      function (c) c:kill() end,
              {description = "close", group = "client"}),


	     
	      
    ---- Focus Only ----



    --- Focus ---

    -- Focus next by index 
    awful.key({ modkey,           }, "h",
        function ()
            awful.client.focus.byidx( 1)
        end,
        {description = "focus next by index", group = "client"}
    ),
    
    -- Focus previous by Index
    awful.key({ modkey,           }, "t",
        function ()
            awful.client.focus.byidx(-1)
        end,
        {description = "focus previous by index", group = "client"}
    ),

    -- Go back
    awful.key({ modkey,           }, "Tab",
        function ()
            awful.client.focus.history.previous()
            if client.focus then
                client.focus:raise()
            end
        end,
        {description = "go back", group = "client"}),




    ---- Layout Manipulation ----

    -- Move to screen 
    awful.key({ modkey,           }, "o",      function (c) c:move_to_screen() end,
              {description = "move to screen", group = "client"}),



    --- Swaping ---
   
    -- Swap with next client by index 
    awful.key({ modkey, "Shift"   }, "h", function () awful.client.swap.byidx(  1) end,
              {description = "swap with next client by index", group = "client"}),

    -- Swap with previous client by index
    awful.key({ modkey, "Shift"   }, "t", function () awful.client.swap.byidx( -1)     end,
              {description = "swap with previous client by index", group = "client"}),

    -- Move to master -- 
    awful.key({ modkey, "Control" }, "Return", function (c) c:swap(awful.client.getmaster()) end,
              {description = "move to master", group = "client"}),



    --- Jumping ---

    -- Jump to urgent client
    awful.key({ modkey,           }, "u", awful.client.urgent.jumpto,
              {description = "jump to urgent client", group = "client"}),
	     


    --- Minimization ---
    
    -- Minimize
    awful.key({ modkey,           }, "w",
        function (c)
            -- The client currently has the input focus, so it cannot be
            -- minimized, since minimized clients can't have the focus.
            c.minimized = true
        end ,
        {description = "minimize", group = "client"}),

    -- Restore minimized
    awful.key({ modkey, "Control" }, "w",
              function ()
                  local c = awful.client.restore()
                  -- Focus restored client
                  if c then
                    c:emit_signal(
                        "request::activate", "key.unminimize", {raise = true}
                    )
                  end
              end,
              {description = "restore minimized", group = "client"}),



   --- Toggle Layouts ---

      
   -- Fullscreen --

   -- Toggle fullscreen
   awful.key({ modkey, "Shift"   }, "m",
        function (c)
            c.fullscreen = not c.fullscreen
            c:raise()
        end,
        {description = "toggle fullscreen", group = "client"}),

	
    -- Maximization --

    -- Toggle Maximization
    awful.key({ modkey,           }, "m",
        function (c)
            c.maximized = not c.maximized
            c:raise()
        end ,
        {description = "(un)maximize", group = "client"}),

    -- Toggle Verticle Maximization 
    awful.key({ modkey, "Control" }, "m",
        function (c)
            c.maximized_vertical = not c.maximized_vertical
            c:raise()
        end ,
        {description = "(un)maximize vertically", group = "client"}),

    -- Toggle Horizontal Maximization	
    awful.key({ modkey, "Shift"   }, "m",
        function (c)
            c.maximized_horizontal = not c.maximized_horizontal
            c:raise()
        end ,
        {description = "(un)maximize horizontally", group = "client"}),


    -- Toggle floating
    awful.key({ modkey, "Control" }, "space",  awful.client.floating.toggle,
              {description = "toggle floating", group = "client"}),

    -- Toggle keep on top
    awful.key({ modkey,           }, "j",      function (c) c.ontop = not c.ontop end,
              {description = "toggle keep on top", group = "client"})

)

    



globalkeys = gears.table.join(globalkeys,

    ----- Screen -----

    -- Focus the next screen 
    awful.key({ modkey, "Control" }, "h", function () awful.screen.focus_relative( 1) end,
              {description = "focus the next screen", group = "screen"}),

    -- Focus the previous screen
    awful.key({ modkey, "Control" }, "t", function () awful.screen.focus_relative(-1) end,
              {description = "focus the previous screen", group = "screen"}),






    ----- Launcher -----
  
    -- Open a terminal
    awful.key({ modkey,           }, "q", function () awful.spawn(terminal) end,
              {description = "open a terminal", group = "launcher"}),

    -- Open the Foot terminal
    awful.key({ modkey, "Control" }, "q", function () awful.spawn("weston --shell=kiosk-shell.so") end,
        {description = "open the Foot terminal", group = "launcher"}),

    -- Open a web browser
    awful.key({ modkey, "Shift"   }, "q",  function () awful.spawn("qutebrowser") end,
              {description = "open a web browser", group = "launcher"}),
    
    -- Take a selection screensot
    awful.key({                   }, "Print", function () awful.util.spawn("flameshot gui") end, 
              {description = "take a selection screenshot", group = "lanchers"}),

    -- Take a screenshot of the entire screen
    awful.key({ "Shift"           }, "Print", function () awful.spawn.with_shell("flameshot screen") end, 
              {description = "take a screenshot of the entire screen", group = "lanchers"}),
 
    -- Run prompt
    awful.key({ modkey },            "r",     function () awful.screen.focused().mypromptbox:run() end,
              {description = "run prompt", group = "launcher"}),

    -- Show the menubar
    awful.key({ modkey }, "p", function() menubar.show() end,
              {description = "show the menubar", group = "launcher"}),


	      
	      
	      
	     
    ----- Tag -----

    -- Veiw Previous
    awful.key({ modkey,           }, "Left",   awful.tag.viewprev,
              {description = "view previous", group = "tag"}),

    -- View Next
    awful.key({ modkey,           }, "Right",  awful.tag.viewnext,
              {description = "view next", group = "tag"}),

    -- Go Back
    awful.key({ modkey,           }, "Escape", awful.tag.history.restore,
              {description = "go back", group = "tag"})

)





---- Number Bindings ----



--- Tags 1-10 ---

for i = 1, 10 do
    local workspace_number = i
    local key = tostring(i + 9)

--  Move focused client to tag 1-10
    globalkeys = gears.table.join(globalkeys,
        awful.key({ "Control" }, "#" .. key,
            function ()
                if client.focus then
                    local tag = client.focus.screen.tags[workspace_number]
                    if tag then
                        client.focus:move_to_tag(tag)
                    end
                end
            end,
            {description = "move focused client to tag #"..workspace_number, group = "tag"})
    )

-- View tag 1-10
    globalkeys = gears.table.join(globalkeys,
        awful.key({ modkey }, "#" .. key,
            function ()
                local screen = awful.screen.focused()
                local tag = screen.tags[workspace_number]
                if tag then
                    tag:view_only()
                end
            end,
            {description = "view tag #"..workspace_number, group = "tag"})
    )
end



--- Tags 11-20 ---

for i = 11, 20 do
    local workspace_number = i
    local key = tostring(i - 1)

--  Move focused client to tag 11-20 
    globalkeys = gears.table.join(globalkeys,
        awful.key({ "Control", "Shift" }, "#" .. key,
            function ()
                if client.focus then
                    local tag = client.focus.screen.tags[workspace_number]
                    if tag then
                        client.focus:move_to_tag(tag)
                    end
                end
            end,
            {description = "move focused client to tag #"..workspace_number, group = "tag"})
    )

-- View tag 11-20
    globalkeys = gears.table.join(globalkeys,
        awful.key({ modkey, "Shift" }, "#" .. key,
            function ()
                local screen = awful.screen.focused()
                local tag = screen.tags[workspace_number]
                if tag then
                    tag:view_only()
                end
            end,
            {description = "view tag #"..workspace_number, group = "tag"})
    )
end






----- SET KEYS -----

root.keys(globalkeys)







----- RULES -----

-- Rules to apply to new clients (through the "manage" signal).

awful.rules.rules = {

    -- All clients will match this rule.
    { rule = { },

      properties = { 

	             border_width = beautiful.border_width,
                     border_color = beautiful.border_normal,
                     focus = awful.client.focus.filter,
                     raise = true,
                     keys = clientkeys,
                     buttons = clientbuttons,
                     screen = awful.screen.preferred,
                     placement = awful.placement.no_overlap+awful.placement.no_offscreen

     }

    },




    ---- Floating Clients ----



    --- Instance ---

    { 

	rule_any = {

        instance = {

          "DTA",  -- Firefox addon DownThemAll.
          "copyq",  -- Includes session name in class.
          "pinentry",

        },


	--- Class ---
	
        class = {

          "Arandr",
          "Blueman-manager",
          "Gpick",
          "Kruler",
          "MessageWin",  -- kalarm.
          "Sxiv",
          "Tor Browser", -- Needs a fixed window size to avoid fingerprinting by screen size.
          "Wpa_gui",
          "veromix",
          "xtightvncviewer"

        },



        --- Name ---
	
        -- Note that the name property shown in xprop might be set slightly after creation of the client
        -- and the name shown there might not match defined rules here.
        name = {

          "Event Tester",  -- xev.

        },


	--- Role
        role = {

          "AlarmWindow",  -- Thunderbird's calendar.
          "ConfigManager",  -- Thunderbird's about:config.
          "pop-up",       -- e.g. Google Chrome's (detached) Developer Tools.

        }



      }, properties = { floating = true }},




    ---- Titlebars ----

    -- Add titlebars to normal clients and dialogs
    { rule_any = {type = { "normal", "dialog" }

      }, properties = { titlebars_enabled = true }

    },




    ---- Screen/Tag Masks ----

    -- Set Firefox to always map on the tag named "2" on screen 1.
    { rule = { class = "Firefox" },

        properties = { screen = 1, tag = "2" } },

}





----- SIGNAL ----



---- Management ----

-- Signal function to execute when a new client appears.
client.connect_signal("manage", function (c)

    -- Set the windows at the slave,
    -- i.e. put it at the end of others instead of setting it master.
    -- if not awesome.startup then awful.client.setslave(c) end
    if awesome.startup
      and not c.size_hints.user_position
      and not c.size_hints.program_position then

        -- Prevent clients from being unreachable after screen count changes.
        awful.placement.no_offscreen(c)
    end

    c.shape = function(cr, w, h, tl, tr, br, bl, rad)
        gears.shape.partially_rounded_rect(cr, w, h, false, true, true, false, 10)
    end

end)

-- Set sharp corners for fullscreen clients  
client.connect_signal("property::fullscreen", function(c)
    if c.fullscreen then
        c.shape = gears.shape.rectangle
    else
        c.shape = function(cr, w, h, tl, tr, br, bl, rad)
            gears.shape.partially_rounded_rect(cr, w, h, false, true, true, false, 10)
        end
    end
end)




---- Titlebars ----

-- Add a titlebar if titlebars_enabled is set to true in the rules.
client.connect_signal("request::titlebars", function(c)



    --- Buttons For The Titlebar ---
    
    local buttons = gears.table.join(
        awful.button({ }, 1, function()
            c:emit_signal("request::activate", "titlebar", {raise = true})
            awful.mouse.client.move(c)
        end),

        awful.button({ }, 3, function()
            c:emit_signal("request::activate", "titlebar", {raise = true})
            awful.mouse.client.resize(c)
        end)
    )


    

    awful.titlebar(c, {size = 20, position = "right"}) : setup {



        --- Top ---
        {

	    {

                { 
	  
                    awful.titlebar.widget.closebutton    (c),
                    awful.titlebar.widget.ontopbutton    (c),
                    awful.titlebar.widget.stickybutton   (c),
                    awful.titlebar.widget.maximizedbutton(c),
                    awful.titlebar.widget.floatingbutton (c),
	   
                    spacing = dpi(7),

                    layout = wibox.layout.fixed.vertical

                },

               layout = wibox.layout.fixed.vertical

            },
        
	top = 5,
	left = 2,
	right = 2,

	widget = wibox.container.margin, 

        },



        --- Middle ---
	
        { 
	
            buttons = buttons,

            layout = wibox.layout.flex.vertical

        },

        


       
	--- Bottom ---
	{

            { 

                { 
	     
                    awful.titlebar.widget.iconwidget(c),

		    layout = wibox.layout.align.vertical

                },

            layout = wibox.layout.align.vertical

    	    },

	bottom = 7,
	left = 2,
	right = 2,

	widget = wibox.container.margin,

        },



    layout = wibox.layout.align.vertical

    }



--- Titlebar-borders ---

    local titlebar_border_size = 5

    awful.titlebar(c, { size = titlebar_border_size, position = 'left'}) : setup {
     
     	 layout = wibox.layout.align.vertical
     }

    awful.titlebar(c, { size = titlebar_border_size, position = 'bottom'}) : setup {

	      layout = wibox.layout.align.horizontal
     }

    awful.titlebar(c, { size = titlebar_border_size, position = 'top'}) : setup {

	      layout = wibox.layout.align.horizontal
     }



end)




---- Focus ----



--- Focus Model --- 

-- Enable sloppy focus, so that focus follows mouse.
client.connect_signal("mouse::enter", function(c)
    c:emit_signal("request::activate", "mouse_enter", {raise = false})
end)



--- Border Color ---

client.connect_signal("focus", function(c) c.border_color = beautiful.border_focus end)
client.connect_signal("unfocus", function(c) c.border_color = beautiful.border_normal end)





----- AUTOSTART -----

-- Pipewire
awful.spawn.with_shell("gentoo-pipewire-launcher")
--awful.spawn.with_shell("pipewire")
--awful.spawn.with_shell("pipewire-pulse")
--awful.spawn.with_shell("wireplumber")

-- Picom
awful.spawn.with_shell("picom -b")

